/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: AnnotationDeploymentAnalyzer.java 1970 2007-10-16 11:49:25Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.deployment.annotations.analyzer;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.Iterator;

import org.objectweb.asm.ClassReader;
import org.ow2.easybeans.deployment.annotations.exceptions.AnalyzerException;
import org.ow2.easybeans.deployment.annotations.metadata.EjbJarAnnotationMetadata;
import org.ow2.util.ee.deploy.api.archive.ArchiveException;
import org.ow2.util.ee.deploy.api.archive.IArchive;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * This class finds the annotated class and fill metadata class.
 * @author Florent Benoit
 */
public class AnnotationDeploymentAnalyzer {

    /**
     * Archive which will be analyzed.
     */
    private IArchive archive = null;

    /**
     * Metadata representing the parsed ejb-jar file.
     */
    private EjbJarAnnotationMetadata ejbJarAnnotationMetadata = null;

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(AnnotationDeploymentAnalyzer.class);

    /**
     * ASM visitor used to visit classes.
     */
    private ScanClassVisitor scanVisitor = null;

    /**
     * Constructor.<br>
     * Archive which will be used when analyzing.
     * @param archive the archive to analyze.
     */
    public AnnotationDeploymentAnalyzer(final IArchive archive) {
        this.archive = archive;
        this.ejbJarAnnotationMetadata = new EjbJarAnnotationMetadata();
        scanVisitor = new ScanClassVisitor(ejbJarAnnotationMetadata);
    }

    /**
     * Analyzes the archive and fill metadata struct.
     * @throws AnalyzerException if analyze of archive fails
     */
    public void analyze() throws AnalyzerException {

        Iterator<URL> iterator;
        try {
            iterator = archive.getResources();
            // Scan all resources
            while (iterator.hasNext()) {
                URL url = iterator.next();
                // only .class
                if (url.getPath().toLowerCase().endsWith(".class")) {
                    InputStream is = null;
                    try {
                        URLConnection urlConnection = url.openConnection();
                        urlConnection.setDefaultUseCaches(false);
                        is = urlConnection.getInputStream();
                        new ClassReader(is).accept(scanVisitor, 0);
                    } catch (Exception ioe) {
                        throw new AnalyzerException("Error while analyzing file entry '" + url + "' in archive '"
                                + archive.getName() + "'", ioe);
                    } finally {
                        if (is != null) {
                            try {
                                is.close();
                            } catch (IOException e) {
                                throw new AnalyzerException("Error while closing input stream of the entry '" + url
                                        + "' in archive '" + archive.getName() + "'", e);
                            }
                        }
                    }
                }
            }

        } catch (ArchiveException e) {
            throw new AnalyzerException("Error while analyzing archive '" + archive.getName() + "'", e);
        } finally {
            archive.close();
        }

    }


    /**
     * @return struct (metadata) filled by this analyzer
     */
    public EjbJarAnnotationMetadata getEjbJarAnnotationMetadata() {
        return ejbJarAnnotationMetadata;
    }

}
